
from kotonebot.tasks.common import sprite_path
from kotonebot.backend.core import Image, HintBox



class Common:
    
    ButtonClose = Image(path=sprite_path(r"e3b5cdcb-58b9-4090-b760-0c98b1615cf1.png"), name="button_close.png")

    ButtonCommuFastforward = Image(path=sprite_path(r"fdb3161a-db55-4bd4-8371-af55f3b0a847.png"), name="button_commu_fastforward.png")

    ButtonCommuSkip = Image(path=sprite_path(r"0a0ed135-3049-4762-9e40-7b709aee3bfd.png"), name="button_commu_skip.png")

    ButtonCompletion = Image(path=sprite_path(r"545f9608-a694-4498-bcad-cd57a365efa8.png"), name="button_completion.png")

    ButtonConfirm = Image(path=sprite_path(r"fd624f2c-8295-4dcd-a496-74eb37370b4d.png"), name="button_confirm.png")

    ButtonConfirmNoIcon = Image(path=sprite_path(r"9ecaa6ee-4986-4c3f-8590-060ecef96057.png"), name="button_confirm_no_icon.png")

    ButtonContest = Image(path=sprite_path(r"a77d19f7-5430-446c-9c23-0fcda7bee78e.png"), name="button_contest.png")

    ButtonEnd = Image(path=sprite_path(r"636f2c68-3215-4d35-9b3d-8bb5c7900604.png"), name="button_end.png")

    ButtonHome = Image(path=sprite_path(r"08b48ec8-3816-4d9e-8dcb-dfba8eb47519.png"), name="button_home.png")

    ButtonIconArrowShort = Image(path=sprite_path(r"7ba14e57-0e8d-449d-bb6b-0e363563cf54.png"), name="button_icon_arrow_short.png")

    ButtonIconArrowShortDisabled = Image(path=sprite_path(r"2cd66720-7afb-4e11-bcef-9c34573160b2.png"), name="button_icon_arrow_short_disabled.png")

    ButtonIconCheckMark = Image(path=sprite_path(r"afd291e2-2910-4f6d-bdb8-508d64c646f8.png"), name="button_icon_check_mark.png")

    ButtonIconClose = Image(path=sprite_path(r"685e8161-17d6-483c-8943-b43d6811079b.png"), name="button_icon_close.png")

    ButtonNext = Image(path=sprite_path(r"7f1f78fe-6a00-4c3a-9b16-bb9df1012eb1.png"), name="button_next.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"db611310-f91b-44c8-8dc1-682d7193adae.png"), name="button_next_no_icon.png")

    ButtonRetry = Image(path=sprite_path(r"79cfdc42-4978-4c67-8ced-79fafe7c9a66.png"), name="button_retry.png")

    ButtonSelect = Image(path=sprite_path(r"d8cea315-74f3-4ec9-a066-606c6974c145.png"), name="button_select.png")

    ButtonStart = Image(path=sprite_path(r"70b96ed0-3c8d-4321-92a6-a6a6f24b2223.png"), name="button_start.png")

    ButtonToolbarHome = Image(path=sprite_path(r"88d57dfb-4c4c-4d1a-b164-277843c9c301.png"), name="button_toolbar_home.png")

    CheckboxUnchecked = Image(path=sprite_path(r"e6a79ffc-acad-476c-b78b-6fcc433a8857.png"), name="checkbox_unchecked.png")

    ButtonToolbarHomeBinary = Image(path=sprite_path(r"2f68e457-4491-4ebc-b436-e3e63db92e14.png"), name="底部工具栏 首页按钮 二值化图片")

    ButtonToolbarMenuBinary = Image(path=sprite_path(r"bf6dcf21-2ffc-441f-82f3-59319f2313b1.png"), name="底部工具栏 菜单按钮 二值化图片")

    TextGameUpdate = Image(path=sprite_path(r"3b301b5a-0658-40ed-8193-f3f8d7d8ca2a.png"), name="text_game_update.png")

    TextNetworkError = Image(path=sprite_path(r"ff789005-f274-45ed-8dea-d9ce58dd0864.png"), name="text_network_error.png")


    pass
class Daily:
    
    ButonLinkData = Image(path=sprite_path(r"dccc72a8-7971-4836-8750-d0cd32cbc4f4.png"), name="buton_link_data.png")

    ButtonAssignmentPartial = Image(path=sprite_path(r"9b1d8702-7462-4a54-b213-5b15961dd6ac.png"), name="button_assignment_partial.png")

    ButtonClaimAllNoIcon = Image(path=sprite_path(r"d6a265c8-77d6-47ef-833f-ff06bf0c2d02.png"), name="button_claim_all_no_icon.png")

    ButtonContestChallenge = Image(path=sprite_path(r"290739f5-3994-476f-a1e5-e2452b5028e2.png"), name="button_contest_challenge.png")

    ButtonContestChallengeStart = Image(path=sprite_path(r"345a0c0b-8b3a-4ba1-983f-b28ed8459dee.png"), name="button_contest_challenge_start.png")

    ButtonContestRanking = Image(path=sprite_path(r"98b89394-b863-495b-8e89-2c3db99d7e7c.png"), name="button_contest_ranking.png")

    ButtonContestStart = Image(path=sprite_path(r"4b961dcf-1189-417c-945e-742e9cb8298a.png"), name="button_contest_start.png")

    ButtonDailyShop = Image(path=sprite_path(r"d9727aa1-50bc-4342-bf69-45294e6ad4dd.png"), name="button_daily_shop.png")

    ButtonHomeCurrent = Image(path=sprite_path(r"65b2fa7c-b4f3-4a89-a58f-b3d69d0ff492.png"), name="button_home_current.png")

    ButtonIconPass = Image(path=sprite_path(r"2b54178e-cebc-4172-a5cd-853ea4ca48b1.png"), name="button_icon_pass.png")

    ButtonIconSkip = Image(path=sprite_path(r"a4232f30-9bf1-4c12-a7aa-8cd68d92b050.png"), name="button_icon_skip.png")

    ButtonMission = Image(path=sprite_path(r"5508aa4a-94b0-44aa-bf7e-eed922835c3c.png"), name="button_mission.png")

    ButtonPass = Image(path=sprite_path(r"2238778e-981a-4a7d-8b93-b469f1dc0a9e.png"), name="button_pass.png")

    ButtonPassClaim = Image(path=sprite_path(r"d94177c8-78f6-4936-b0d7-5b038066d9fa.png"), name="button_pass_claim.png")

    ButtonPresentsPartial = Image(path=sprite_path(r"6d693394-cf66-4fb7-bb4a-711fe115f06d.png"), name="button_presents_partial.png")

    ButtonProduce = Image(path=sprite_path(r"942a0128-14ec-4ab4-9b2e-e1511a0ab207.png"), name="button_produce.png")

    ButtonShop = Image(path=sprite_path(r"beae8cfe-7320-40bf-85e9-53dc2b7e77c6.png"), name="button_shop.png")

    ButtonShopCountAdd = Image(path=sprite_path(r"cdf86260-9c8d-4389-8137-8583ad476d4b.png"), name="button_shop_count_add.png")

    ButtonShopCountAddDisabled = Image(path=sprite_path(r"e9281664-7c68-4963-9c28-1416ccc9be84.png"), name="button_shop_count_add_disabled.png")

    IconTitleDailyShop = Image(path=sprite_path(r"e9ee330d-dfca-440e-8b8c-0a3b4e8c8730.png"), name="日常商店标题图标")

    BoxHomeAssignment = HintBox(x1=16, y1=642, x2=127, y2=752, source_resolution=(720, 1280))

    BoxHomeAP = HintBox(x1=291, y1=4, x2=500, y2=82, source_resolution=(720, 1280))

    BoxHomeJewel = HintBox(x1=500, y1=7, x2=703, y2=82, source_resolution=(720, 1280))

    BoxHomeActivelyFunds = HintBox(x1=11, y1=517, x2=137, y2=637, source_resolution=(720, 1280))

    IconAssignKouchou = Image(path=sprite_path(r"2f9a0052-8613-4c9d-bbcd-97a0ee784224.png"), name="icon_assign_kouchou.png")

    IconAssignMiniLive = Image(path=sprite_path(r"e8185b4d-3f72-4736-9842-9ef4c06dc252.png"), name="icon_assign_mini_live.png")

    IconAssignOnlineLive = Image(path=sprite_path(r"0f4c13c0-8cec-401a-bcce-818218314211.png"), name="icon_assign_online_live.png")

    IconAssignTitle = Image(path=sprite_path(r"a384caea-3fbb-4d44-99f7-daffbc83c9ce.png"), name="icon_assign_title.png")

    IconShopAp = Image(path=sprite_path(r"4eb8735c-c8de-403f-94a5-fb14a4764e81.png"), name="icon_shop_ap.png")

    IconShopMoney = Image(path=sprite_path(r"bb1e4fc9-e16a-4694-a2b2-9f5ea2e4b35b.png"), name="icon_shop_money.png")

    IconShopTitle = Image(path=sprite_path(r"80e2682d-6a9f-42d1-8c0d-5fac5b82d17a.png"), name="icon_shop_title.png")

    IconTitleAssign = Image(path=sprite_path(r"6a2ddc90-1bb6-4fcb-91d4-e650e5c96254.png"), name="icon_title_assign.png")

    IconTitlePass = Image(path=sprite_path(r"c333d35e-46bb-4bdf-8685-0e2f06191dde.png"), name="icon_title_pass.png")

    ButtonAssignmentShortenTime = Image(path=sprite_path(r"1652f06a-5417-49ef-8949-4854772d9ab7.png"), name="工作页面 短缩 时间")

    TextActivityFundsMax = Image(path=sprite_path(r"09f4278a-b95d-40c9-a083-6072c45dc8e7.png"), name="text_activity_funds_max.png")

    TextAssignmentCompleted = Image(path=sprite_path(r"e533bb6b-8dce-4ed2-abcb-a454c3ded7e7.png"), name="text_assignment_completed.png")

    TextContest = Image(path=sprite_path(r"888fe566-7ec4-4158-9ffe-12beccc184a8.png"), name="text_contest.png")

    TextContestLastOngoing = Image(path=sprite_path(r"1ce41091-4235-46f0-a1ad-b6476a7e0b8e.png"), name="text_contest_last_ongoing.png")

    TextContestNoMemory = Image(path=sprite_path(r"22159eb3-c923-42e6-ada5-f6abeee38162.png"), name="text_contest_no_memory.png")

    TextContestOverallStats = Image(path=sprite_path(r"6848cd81-b7b6-4665-8efa-44be000d7892.png"), name="text_contest_overall_stats.png")

    TextShopPurchased = Image(path=sprite_path(r"13cd2e0a-9428-46c0-a184-1289cb2e094a.png"), name="text_shop_purchased.png")

    TextShopRecommended = Image(path=sprite_path(r"532f3468-64e2-4301-b189-951c78fc9ff2.png"), name="text_shop_recommended.png")

    TextTabShopAp = Image(path=sprite_path(r"e832e34d-eec6-4655-8df4-bc50c1a35888.png"), name="text_tab_shop_ap.png")


    pass
class Shop:
    
    ItemLessonNote = Image(path=sprite_path(r"0949c622-9067-4f0d-bac2-3f938a1d2ed2.png"), name="レッスンノート")

    ItemVeteranNote = Image(path=sprite_path(r"b2af59e9-60e3-4d97-8c72-c7ba092113a3.png"), name="ベテランノート")

    ItemSupportEnhancementPt = Image(path=sprite_path(r"835489e2-b29b-426c-b4c9-3bb9f8eb6195.png"), name="サポート強化Pt 支援强化Pt")

    ItemSenseNoteVocal = Image(path=sprite_path(r"c5b7d67e-7260-4f08-a0e9-4d31ce9bbecf.png"), name="センスノート（ボーカル）感性笔记（声乐）")

    ItemSenseNoteDance = Image(path=sprite_path(r"0f7d581d-cea3-4039-9205-732e4cd29293.png"), name="センスノート（ダンス）感性笔记（舞蹈）")

    ItemSenseNoteVisual = Image(path=sprite_path(r"d3cc3323-51af-4882-ae12-49e7384b746d.png"), name="センスノート（ビジュアル）感性笔记（形象）")

    ItemLogicNoteVocal = Image(path=sprite_path(r"a1df3af1-a3e7-4521-a085-e4dc3cd1cc57.png"), name="ロジックノート（ボーカル）理性笔记（声乐）")

    ItemLogicNoteDance = Image(path=sprite_path(r"a9fcaf04-0c1f-4b0f-bb5b-ede9da96180f.png"), name="ロジックノート（ダンス）理性笔记（舞蹈）")

    ItemLogicNoteVisual = Image(path=sprite_path(r"c3f536d6-a04a-4651-b3f9-dd2c22593f7f.png"), name="ロジックノート（ビジュアル）理性笔记（形象）")

    ItemAnomalyNoteVocal = Image(path=sprite_path(r"eef25cf9-afd0-43b1-b9c5-fbd997bd5fe0.png"), name="アノマリーノート（ボーカル）非凡笔记（声乐）")

    ItemAnomalyNoteDance = Image(path=sprite_path(r"df991b42-ed8e-4f2c-bf0c-aa7522f147b6.png"), name="アノマリーノート（ダンス）非凡笔记（舞蹈）")

    ItemAnomalyNoteVisual = Image(path=sprite_path(r"9340b854-025c-40da-9387-385d38433bef.png"), name="アノマリーノート（ビジュアル）非凡笔记（形象）")

    ItemRechallengeTicket = Image(path=sprite_path(r"ea1ba124-9cb3-4427-969a-bacd47e7d920.png"), name="再挑戦チケット 重新挑战券")

    ItemRecordKey = Image(path=sprite_path(r"1926f2f9-4bd7-48eb-9eba-28ec4efb0606.png"), name="記録の鍵  解锁交流的物品")

    class IdolPiece:
        
        倉本千奈_WonderScale = Image(path=sprite_path(r"6720b6e8-ae80-4cc0-a885-518efe12b707.png"), name="倉本千奈 WonderScale 碎片")
    
        篠泽广_光景 = Image(path=sprite_path(r"afa06fdc-a345-4384-b25d-b16540830256.png"), name="篠泽广 光景 碎片")
    
        紫云清夏_TameLieOneStep = Image(path=sprite_path(r"278b7d9c-707e-4392-9677-74574b5cdf42.png"), name="紫云清夏 Tame-Lie-One-Step 碎片")
    
    
        pass

    pass
class Produce:
    
    BoxProduceOngoing = HintBox(x1=179, y1=937, x2=551, y2=1091, source_resolution=(720, 1280))

    ButtonAutoSet = Image(path=sprite_path(r"d03ea7b6-da57-4a40-8e92-b4a4bbce0bb8.png"), name="button_auto_set.png")

    ButtonProduce = Image(path=sprite_path(r"c19640c8-3f23-4929-9def-1b6d7eb7c91a.png"), name="button_produce.png")

    ButtonProduceStart = Image(path=sprite_path(r"b8481e1f-75bb-452f-9b5a-d50e330a60a6.png"), name="button_produce_start.png")

    ButtonRegular = Image(path=sprite_path(r"7ad99675-e3ff-4fa6-9d29-9cabbfe3cf40.png"), name="button_regular.png")

    CheckboxIconNoteBoost = Image(path=sprite_path(r"a1a0a94f-7214-4a0b-9899-e6fc64e3a82c.png"), name="checkbox_icon_note_boost.png")

    CheckboxIconSupportPtBoost = Image(path=sprite_path(r"e3b62d4e-0bbf-44b4-b28a-1a4bff9d2b2f.png"), name="checkbox_icon_support_pt_boost.png")

    BoxModeButtons = HintBox(x1=7, y1=818, x2=713, y2=996, source_resolution=(720, 1280))

    ButtonPIdolOverview = Image(path=sprite_path(r"e88c9ad1-ec37-4fcd-b086-862e1e7ce8fd.png"), name="Pアイドルー覧  P偶像列表展示")

    BoxStepIndicator = HintBox(x1=4, y1=11, x2=405, y2=99, source_resolution=(720, 1280))

    IconPIdolLevel = Image(path=sprite_path(r"30a6f399-6999-4f04-bb77-651e0214112f.png"), name="P偶像卡上的等级图标")

    KbIdolOverviewName = HintBox(x1=140, y1=16, x2=615, y2=97, source_resolution=(720, 1280))

    ButtonResume = Image(path=sprite_path(r"ccbcb114-7f73-43d1-904a-3a7ae660c531.png"), name="再開する")

    BoxResumeDialogProduceType = HintBox(x1=186, y1=491, x2=541, y2=550, source_resolution=(720, 1280))

    BoxResumeDialogIdolTitle = HintBox(x1=194, y1=794, x2=523, y2=840, source_resolution=(720, 1280))

    RadioTextSkipCommu = Image(path=sprite_path(r"ab56463b-a268-4b59-89cf-c180160204f0.png"), name="radio_text_skip_commu.png")

    TextAPInsufficient = Image(path=sprite_path(r"4883c564-f950-4a29-9f5f-6f924123cd22.png"), name="培育 AP 不足提示弹窗 标题")

    ButtonRefillAP = Image(path=sprite_path(r"eaba6ebe-f0df-4918-aee5-ef4e3ffedcf0.png"), name="确认恢复AP按钮")

    ScreenshotNoEnoughAp2 = Image(path=sprite_path(r"cc673f49-eee6-47af-ae8c-e99726776ec0.png"), name="screenshot_no_enough_ap_2.png")

    TextRentAvailable = Image(path=sprite_path(r"af561656-5214-4d0d-aa9b-eb57e275b224.png"), name="text_rent_available.png")


    pass
class InPurodyuusu:
    
    A = Image(path=sprite_path(r"b9c5e83e-4f5f-4813-9b82-932867065383.png"), name="A.png")

    AcquireBtnDisabled = Image(path=sprite_path(r"e65fd66c-0fc3-4090-8d55-af50b753c0d0.png"), name="acquire_btn_disabled.png")

    ButtonCancel = Image(path=sprite_path(r"74fc7bf9-0079-4bbe-a264-7a6077de698a.png"), name="button_cancel.png")

    ButtonComplete = Image(path=sprite_path(r"f6b5bc5b-ec13-469b-87c5-80601b5e475d.png"), name="button_complete.png")

    ButtonFinalPracticeDance = Image(path=sprite_path(r"68def82e-472e-4fe9-9148-2d68324ee7eb.png"), name="button_final_practice_dance.png")

    ButtonFinalPracticeVisual = Image(path=sprite_path(r"04f9aed6-d542-4a11-999d-77adb45e4cc2.png"), name="button_final_practice_visual.png")

    ButtonFinalPracticeVocal = Image(path=sprite_path(r"e0fdc58d-8602-4649-ad3b-3ab0941e09e4.png"), name="button_final_practice_vocal.png")

    ButtonFollowNoIcon = Image(path=sprite_path(r"7dfbb44e-a169-40be-8f16-c719e9b66b7d.png"), name="button_follow_no_icon.png")

    ButtonIconStudy = Image(path=sprite_path(r"302dcb9b-ceb1-4df1-87d5-0059b2f3360c.png"), name="button_icon_study.png")

    ButtonIconStudyVisual = Image(path=sprite_path(r"963c4066-ba01-4e6e-9818-91835eaf1711.png"), name="button_icon_study_visual.png")

    ButtonLeave = Image(path=sprite_path(r"dc2a50fe-e0aa-4972-901a-ba73290b592f.png"), name="button_leave.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"4c1de807-85ea-45e6-86be-509567dac54e.png"), name="button_next_no_icon.png")

    ButtonNotAcquire = Image(path=sprite_path(r"9e63b633-a7c2-4c93-8612-b5edfee49f2d.png"), name="button_not_acquire.png")

    ButtonRetry = Image(path=sprite_path(r"4eaa3665-d81a-4e75-aa1e-8b3523649e65.png"), name="button_retry.png")

    ButtonTextActionOuting = Image(path=sprite_path(r"c7bfe369-8a93-4006-a362-a4dd8dcc9b7c.png"), name="button_text_action_outing.png")

    ButtonTextAllowance = Image(path=sprite_path(r"60f98b5d-cfe5-459c-a9c2-8303a89cb70c.png"), name="button_text_allowance.png")

    ButtonTextConsult = Image(path=sprite_path(r"7bf6e99b-333d-4b40-b7b8-28d053aa592d.png"), name="button_text_consult.png")

    IconClearBlue = Image(path=sprite_path(r"77180382-5c9b-4d38-9477-9a42775abf8f.png"), name="icon_clear_blue.png")

    IconTitleAllowance = Image(path=sprite_path(r"3f62e201-8a35-43dd-b182-a9c6d7c39a37.png"), name="icon_title_allowance.png")

    IconTitleStudy = Image(path=sprite_path(r"f04e6279-a2ad-44fa-a346-f2f132cd3e6b.png"), name="icon_title_study.png")

    LootboxSliverLock = Image(path=sprite_path(r"30bbd6d3-eb46-43b3-9434-606d4ff4a642.png"), name="lootbox_sliver_lock.png")

    LootBoxSkillCard = Image(path=sprite_path(r"347ec27e-8f6a-4b28-a08f-8455d5a371ad.png"), name="loot_box_skill_card.png")

    M = Image(path=sprite_path(r"543b5354-002d-422c-8346-7b70bdbd4bda.png"), name="M.png")

    BoxWeeksUntilExam = HintBox(x1=11, y1=8, x2=237, y2=196, source_resolution=(720, 1280))

    BoxExamTop = HintBox(x1=5, y1=2, x2=712, y2=55, source_resolution=(720, 1280))

    PDrinkIcon = Image(path=sprite_path(r"c93b7555-2445-4452-9725-76ba274cfe93.png"), name="p_drink_icon.png")

    PItemIconColorful = Image(path=sprite_path(r"e5699f8d-d853-491f-9565-67219dfcc58e.png"), name="p_item_icon_colorful.png")

    PSkillCardIconBlue = Image(path=sprite_path(r"7bdb8a9f-0dbc-4c92-aceb-24c74c46eeb3.png"), name="p_skill_card_icon_blue.png")

    PSkillCardIconColorful = Image(path=sprite_path(r"cf38fe4e-5ae8-44c2-a03d-bc81a689a270.png"), name="p_skill_card_icon_colorful.png")

    Rest = Image(path=sprite_path(r"6bc6cd1c-096b-49fd-bec7-c5197c5d6f75.png"), name="rest.png")

    RestConfirmBtn = Image(path=sprite_path(r"a1f0f268-2b46-4828-82cd-96b4d1b038ec.png"), name="rest_confirm_btn.png")

    BoxNoSkillCard = HintBox(x1=180, y1=977, x2=529, y2=1026, source_resolution=(720, 1280))

    BoxPDrinkMaxConfirmTitle = HintBox(x1=46, y1=829, x2=270, y2=876, source_resolution=(720, 1280))

    BoxSelectPStuff = HintBox(x1=62, y1=558, x2=655, y2=716, source_resolution=(720, 1280))

    BoxSelectPStuffComfirm = HintBox(x1=256, y1=1064, x2=478, y2=1128, source_resolution=(720, 1280))

    BoxSkillCardAcquired = HintBox(x1=194, y1=712, x2=528, y2=765, source_resolution=(720, 1280))

    BoxSkillCardEnhaced = HintBox(x1=49, y1=948, x2=676, y2=1106, source_resolution=(720, 1280))

    BoxCommuEventButtonsArea = HintBox(x1=14, y1=412, x2=703, y2=1089, source_resolution=(720, 1280))

    TextAsariProduceEnd = Image(path=sprite_path(r"55972f3f-d576-45b6-9c4e-2ad8d46200ef.png"), name="text_asari_produce_end.png")

    TextButtonExamSkipTurn = Image(path=sprite_path(r"214efc0e-70ac-418f-a6ed-bcc793e108f9.png"), name="text_button_exam_skip_turn.png")

    TextClearUntil = Image(path=sprite_path(r"9b3e0f9f-0cd5-47db-89f2-9621d3dc66cf.png"), name="text_clear_until.png")

    TextDance = Image(path=sprite_path(r"06c22a4a-3d77-479a-8e75-f9ce12f2d927.png"), name="text_dance.png")

    TextFinalProduceRating = Image(path=sprite_path(r"fabc1f1e-a6c5-4b6d-b895-ca7a7968ef6e.png"), name="text_final_produce_rating.png")

    TextOneWeekRemaining = Image(path=sprite_path(r"c00ef56a-b994-4766-b99d-4e7a88f6c88c.png"), name="text_one_week_remaining.png")

    TextPerfectUntil = Image(path=sprite_path(r"f6272354-1e9e-4d4e-b679-78943bf3f4da.png"), name="text_perfect_until.png")

    TextPleaseSelectPDrink = Image(path=sprite_path(r"04f987f0-4056-48ef-ba72-9cc596b81369.png"), name="text_please_select_p_drink.png")

    TextPDiary = Image(path=sprite_path(r"573a4bbe-9458-44ba-8c35-d0e36100a9e1.png"), name="text_p_diary.png")

    TextPDrinkMax = Image(path=sprite_path(r"d87c8434-24a1-4ec6-95bf-ec74403d7708.png"), name="text_p_drink_max.png")

    TextSenseiTipConsult = Image(path=sprite_path(r"635c6e8e-7079-42fe-b9a0-2c3b4b48c076.png"), name="text_sensei_tip_consult.png")

    TextSenseiTipDance = Image(path=sprite_path(r"775451ff-63a5-4d1b-b47c-842d615ec266.png"), name="text_sensei_tip_dance.png")

    TextSenseiTipRest = Image(path=sprite_path(r"a779eb78-6aec-47e9-85e1-9dff881644b9.png"), name="text_sensei_tip_rest.png")

    TextSenseiTipVisual = Image(path=sprite_path(r"96adda6c-19e7-4109-b541-e84ca7df8562.png"), name="text_sensei_tip_visual.png")

    TextSenseiTipVocal = Image(path=sprite_path(r"8b35d64f-34e3-4969-9552-b7da0644dbef.png"), name="text_sensei_tip_vocal.png")

    TextSkipTurnDialog = Image(path=sprite_path(r"dd9b55bc-dc79-4f2f-a53a-a6f796711d65.png"), name="text_skip_turn_dialog.png")

    TextVisual = Image(path=sprite_path(r"80effacb-5f16-4fcf-a151-581f97475393.png"), name="text_visual.png")

    class Action:
        
        ActionStudy = Image(path=sprite_path(r"44f8fb09-1b51-4178-9180-6bd34ae534e1.png"), name="action_study.png")
    
        PDorinkuBg = Image(path=sprite_path(r"0eb0ca65-9c2d-4eb6-9b30-9dd423c46204.png"), name="p_dorinku_bg.png")
    
        PDorinkuBgMask = Image(path=sprite_path(r"ca629da2-0655-4e68-9bf0-5eb40a64834d.png"), name="p_dorinku_bg_mask.png")
    
        PItem = Image(path=sprite_path(r"fd0c8246-fb6b-48a7-bf98-49954458bc81.png"), name="p_item.png")
    
        VocalWhiteBg = Image(path=sprite_path(r"d664f6b6-62fe-4542-9ac5-23ea006b9eb9.png"), name="vocal_white_bg.png")
    
    
        pass

    pass